/* rsa.c -- berekent a^b mod c waarbij a,b en c grote base-26 getallen zijn.

   Geschreven door Gerton Lunter, 28/5/97, voor de programmeerwedstrijd
   in oktober '97. */

#include <stdio.h>
#define maxdig 82

typedef char getal[maxdig];

getal macht[2*maxdig];              // macht[i] == 26^i modulo mod
getal mod;                          // modulus
getal nul;                          // lijkt me duidelijk

char deel2(getal x)                 // x wordt x/2; levert x modulo 2
{
    int i=maxdig;
    char c=0;
    while (i) {
        i--;
	x[i] = (c = (x[i]+26*(c&1))) >> 1;
    }
    return (c&1);
}

char vergelijk(getal a,getal b)     // levert 1 als a>b, -1 als a<b, anders 0
{
    int i=maxdig;
    while (i) {
        i--;
        if (a[i]>b[i])
            return 1;
        if (a[i]<b[i])
            return -1;
    }
    return 0;
}

void trekaf(getal a, getal b, getal c)      // c wordt a-b
{
    int i=0;
    int carry=0;
    while (i<maxdig) {
        c[i] = a[i]-b[i]+carry;
        if (c[i]<0) {
            c[i]+=26;
            carry = -1;
        } else
          carry=0;
        i++;
    }
}

void init(void)                             // initialiseert macht-array
{
    int i,j;
    for (i=0;i<maxdig;i++) 
        macht[0][i]=0;
    macht[0][0]=1;
    for (j=1;j<2*maxdig;j++) {
        for (i=1;i<maxdig;i++)
            macht[j][i] = macht[j-1][i-1];
        macht[j][0] = 0;
        while (vergelijk(macht[j],mod) >= 0)
            trekaf(macht[j],mod,macht[j]);
    }
}

void keer(getal a, getal b, getal c)        // c wordt a*b modulo mod
{
    unsigned int tussen[2*maxdig];
    unsigned int x;
    char verandering;
    int i,j;

    // doen

    tussen[0]=0;
    for (i=0;i<2*maxdig-1;i++) {
        for (j=0;j<maxdig;j++) {
            if ((i-j>=0) && (i-j<maxdig))
                tussen[i] += a[i-j]*b[j];
        }
        tussen[i+1] = tussen[i]/26;
        tussen[i] %= 26;
    }

    // nu modulus nemen & carry rippelen net zolang tot er niets meer verandert

    do {
        verandering = 0;
        for (i=0;i<2*maxdig;i++) {
            x=tussen[i];
            if (x && ((i>=maxdig) || (!macht[i][i]))) {
	      tussen[i]=0;
              verandering=1;
	      for (j=0;j<maxdig;j++)
		tussen[j] += x*(unsigned int)macht[i][j];
	    }
	}
        for (i=0;i<2*maxdig-1;i++) {
            if (tussen[i]/26)
                verandering=1;
            tussen[i+1] += tussen[i]/26;
            tussen[i] %= 26;
        }
    } while (verandering);

    // resultaat opslaan & meest significante digit "moduleren"

    for (i=0;i<maxdig;i++)
        c[i]=tussen[i];
    while (vergelijk(c,mod) >= 0)
          trekaf(c,mod,c);

}


void neemmacht(getal a, getal b, getal c)   // c wordt a^b modulo mod
{
    getal atot2toti;
    int i;

    for (i=0;i<maxdig;i++) {
        c[i]=0;
        atot2toti[i]=a[i];
    }
    c[0]=1;

    while (vergelijk(b,nul)) {
        if (deel2(b))
            keer(c,atot2toti,c);
        keer(atot2toti,atot2toti,atot2toti);
    }
}


void leesgetal(getal a)
{
    int i=maxdig,j;
    char c;
    do {
      scanf("%c",&c);
      a[--i] = c-'A';
    } while (c!='\n');
    j=0;
    do {
      a[j++]=a[++i];
    } while (i<maxdig-1);
    do {
      a[j++]=0;
    } while (j<maxdig);
}


void schrijfgetal(getal a)
{
	 int i=maxdig;
	 do {
		  i--;
	 } while (i && !a[i]);
	 do {
		  printf ("%c",a[i--]+'A');
	 } while (i+1);
	 printf ("\n");
}


void main(void)
{
	 getal a,b,c;
	 
	 {
	   int i=0;
	   for (;i<maxdig;)
	     nul[i++]=0;
	 }
	 leesgetal(mod);
	 init();
	 while (vergelijk(mod,nul)) {
	   init();
	   leesgetal(a);
	   leesgetal(b);
	   neemmacht(a,b,c);
	   schrijfgetal(c);
	   leesgetal(mod);
	 }
}
