program Manhattan (input, output);

const
   max = 100;

type
   { or\"ientatie co\"ordinaten:
    ---> 2e dim. <==> lanes
    |
    |
    1e dim. <==> avenues }
   rooster = array[1..max, 1..max] of boolean;
      { true <--> kruispunt toegankelijk
       false <--> kruispunt afgesloten }
   coord = record
      avenue, lane : integer;
   end;
   telmatrix = array[1..max, 1..max] of integer;

var
   stad, nsteden,          { run-variabelen }
   navenues,               { #avenues }
   nlanes,                 { #lanes }
   maxstap : integer;      { maximaal #stappen }
   kaart : rooster;
   bereikbaar : telmatrix;


procedure leesKaart (var navenues, nlanes, maxstap : integer; var kaart : rooster);
{ lees kaart in en sla die op }

var
   ch   : char;
   a, l : integer;     { avenues & lanes }

begin
   readln(navenues, nlanes, maxstap);
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 read(ch);
	 kaart[a, l] := (ch='+');
      end;
      readln;
   end;
end; { leesKaart }


function viaAvenue (pos : coord; var naar : coord) : boolean;
{ checkt of er een andere lane via een avenue te bereiken valt
 en zoja, welke }

begin
   if odd(pos.avenue) then begin
      if pos.lane > 1 then begin
	 naar.avenue := pos.avenue;
	 naar.lane := pos.lane-1;
	 viaAvenue := kaart[naar.avenue, naar.lane];
      end else begin
	 viaAvenue := false;
      end;
   end else begin
      if pos.lane < nlanes then begin
	 naar.avenue := pos.avenue;
	 naar.lane := pos.lane+1;
	 viaAvenue := kaart[naar.avenue, naar.lane];
      end else begin
	 viaAvenue := false;
      end;
   end;
end; { viaAvenue }


function viaLane (pos : coord; var naar : coord) : boolean;
{ checkt of er een andere avenue via een lane te bereiken valt
 en zoja, welke }

begin
   if odd(pos.lane) then begin
      if pos.avenue < navenues then begin
	 naar.avenue := pos.avenue+1;
	 naar.lane := pos.lane;
	 viaLane := kaart[naar.avenue, naar.lane];
      end else begin
	 viaLane := false;
      end;
   end else begin
      if pos.avenue > 1 then begin
	 naar.avenue := pos.avenue-1;
	 naar.lane := pos.lane;
	 viaLane := kaart[naar.avenue, naar.lane];
      end else begin
	 viaLane := false;
      end;
   end;
end; { viaLane }


procedure wandel (a, l : integer; var bereikbaar : telmatrix);

const
   kruismax = 10000;

var
   queue : array[1..kruismax] of coord;
   gehad : array[1..max, 1..max] of boolean;
   stap,                  { lengte van het pad }
   beginQ,                { index begin queue }
   eindQ,                 { index einde queue }
   lengteQ,               { lengte nieuwe queue }
   i, j, p : integer;     { hulp-variabelen }
   naar : coord;          { hulp-variabele }

begin
   { initialiseer gehad-matrix }
   for i := 1 to navenues do begin
      for j := 1 to nlanes do begin
	 gehad[i, j] := false;
      end;
   end;
   gehad[a, l] := true;  { niet meer bij langslopen }
   { initialiseer queue door huidige kruispunt erop te plaatsen }
   stap := 0;
   beginQ := 1;
   eindQ := 1;
   lengteQ := 1;
   queue[1].avenue := a;
   queue[1].lane := l;
   while (lengteQ > 0) and (stap < maxstap) do begin
      lengteQ := 0;
      stap := stap+1;
      for p := beginQ to eindQ do begin
	 { haal kruispunt van de queue en ga naar aangrenzende kruispunten }
	 if viaAvenue(queue[p], naar) then begin
	    if not gehad[naar.avenue, naar.lane] then begin
	       gehad[naar.avenue, naar.lane] := true;
	       lengteQ := lengteQ+1;
	       queue[eindQ+lengteQ] := naar;
	    end;
	 end;
	 if viaLane(queue[p], naar) then begin
	    if not gehad[naar.avenue, naar.lane] then begin
	       gehad[naar.avenue, naar.lane] := true;
	       lengteQ := lengteQ+1;
	       queue[eindQ+lengteQ] := naar;
	    end;
	 end;
      end;
      { verplaats queue-indices }
      beginQ := eindQ + 1;
      eindQ := beginQ + lengteQ - 1;
   end;
   { werk statistieken bij }
   for i := 1 to navenues do begin
      for j := 1 to nlanes do begin
	 if gehad[i, j] then begin
	    bereikbaar[i, j] := bereikbaar[i, j]+1;
	 end;
      end;
   end;
   bereikbaar[a, l] := bereikbaar[a, l]-1;   { correctie }
end; { wandel }


procedure wandelAlle (var bereikbaar : telmatrix);
{ ga alle bereikbare kruispunten af: alle-paren kortste pad }

var
   a, l : integer;  { loop-variabelen voor avenues & lanes }

begin
   { initialiseer statistieken }
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 bereikbaar[a, l] := 0;
      end;
   end;
   { voor alle kruispunten: ... }
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 if kaart[a, l] then begin
	    wandel(a, l, bereikbaar);
	 end;
      end;
   end;
end; { wandelAlle }
  

procedure schrijfHistogram;

var
   a, l,                 { loop-variabelen voor avenues & lanes }
   b, max : integer;     { (., maximale) bereikbaarheid }

begin
   { bepaal maximale bereikbaarheid }
   max := 0;
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 if bereikbaar[a, l]>max then begin
	    max := bereikbaar[a, l];
	 end;
      end;
   end;
   { druk af }
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 if not kaart[a, l] then begin
	    write('x');
	 end else begin
	    b := bereikbaar[a, l];
	    if b = 0 then begin
	       write('Z');
	    end else begin
	       if 3*b < max then begin
		  write('R');
	       end else begin
		  if 3*b >= 2*max then begin
		     write('G');
		  end else begin
		     write('O');
		  end;
	       end;
	    end;
	 end;
      end;
      writeln;
   end;
end; { schrijfHistogram }



begin
   readln(nsteden);
   for stad := 1 to nsteden do begin
      leesKaart(navenues, nlanes, maxstap, kaart);
      wandelAlle(bereikbaar);
      schrijfHistogram;
      if stad<nsteden then begin
	 writeln;
      end;
   end;
end.
