/*
   turing.c -- Turing-compiler
   (c) 10/10/'97 Meinte Boersma
   (a) 11/10/'97 Meinte Boersma
*/

#include <stdio.h>

#define maxstate 10000

#define alphabet(ord) ((ord!=7)?(ord+'a'):'~')
#define inv_alphabet(symb) ((symb!='~')?(symb-'a'):7)

typedef struct {
  int newstate;
  char newsymb;
  char dir;
} transsymb;
/* holds transition-description for one alphabet-character */

int nstates = 0;
/* number of states (I ==> nstates points to first free state) */
transsymb transfunc[maxstate][8];


char nextchar (void)
{
  char ch;

  while(scanf("%c", &ch), ((ch == ' ') || (ch == '\n'))) ;
  return(ch);
}


void compile_direction (char dir)
     /* post: "direction" compiled to state (nstates-1) */
{
  int i;
  transsymb *p;

  for (i=0; i<8; i++) {
    p = &(transfunc[nstates][i]);
    p->newstate = nstates+1;
    p->newsymb = alphabet(i);
    p->dir = dir;
  }
  nstates++;
}


void compile_halt (void)
     /* post: "halt" compiled to state (nstates-1) */
{
  int i;
  transsymb *p;

  for (i=0; i<8; i++) {
    p = &(transfunc[nstates][i]);
    p->newstate = nstates;
    p->newsymb = alphabet(i);
    p->dir = 'h';
  }
  nstates++;
}


void compile_write (void)
     /* post: "write" compiled to state (nstates-1) */
{
  char symb, dir, ch;
  int i;
  transsymb *p;
  
  ch = nextchar();   /* == '(' */
  symb = nextchar();
  ch = nextchar();   /* == ',' */
  dir = nextchar();
  ch = nextchar();   /* == ')' */
  for (i=0; i<8; i++) {
    p = &(transfunc[nstates][i]);
    p->newstate = nstates+1;
    p->newsymb = symb;
    p->dir = dir;
  }
  nstates++;
}


/* forward declaration: */
void compile (void);


void compile_loop (void)
     /* pre:  stack = nstates
	post: "loop" compiled to states stack..(nstates-1)
     */
{
  char symb, ch;
  int branch;

  int i;
  transsymb *p;

  ch = nextchar();   /* == '(' */
  symb = nextchar();
  ch = nextchar();   /* == ',' */
  /* reserve loop-header/branch-state and compile */
  branch = nstates;
  nstates++;
  compile();
  /* modify last frag-state to return to state stack */
  for (i=0; i<8; i++) {
    transfunc[nstates-1][i].newstate = branch;
  }
  /* make branch-state */
  for (i=0; i<8; i++) {
    p = &(transfunc[branch][i]);
    p->newstate = branch+1;
    p->newsymb = alphabet(i);
    p->dir = '.';
  }
  p = &(transfunc[branch][inv_alphabet(symb)]);
  p->newstate = nstates;
/*p->newsymb = symb;*/
  p->dir = '.';
  /* make exit-state (necessary for double loops) */
  compile_direction('.');
}


void compile (void)
     /* pre: stdin contains syntactically correct part of PANP-program
	post: part of program (upto ';') compiled to states 0..(nstates-1)
     */
{
  char instr, ch;
  
  instr = nextchar();
  switch (instr) {
    case 'h' : compile_halt(); break;
    case 'w' : compile_write(); break;
    case 'l' : compile_loop(); break;
    case 'L' : case 'R' : case '.' :
      compile_direction(instr); break;
  } /* post: previous part of program compiled to states 0..(nstates-1) with
             exit to state nstates */
  ch = nextchar();
  switch (ch) {
    case '!' : compile_halt(); break;
    case ';' : compile(); break;     /* continue compiling recursively */
    case ')' : break;                /* jump back to parent-level nesting */
  }
}


int main (void)
{
  int state;

  int i;
  transsymb *p;

  compile();

  printf(" q      a      b      c      d      e      f      g      ~\n");
  for (state=0; state<nstates; state++) {
    printf("%4d", state);
    for (i=0; i<8; i++) {
      p = &(transfunc[state][i]);
      printf(" %4d%c%c", p->newstate, p->newsymb, p->dir);
    }
    printf("\n");
  }
  printf("*\n");

  return(0);
}
