program TMsimulator (input, output, transfile, inputfile);
{ checkt syntax en (semantische) correctheid van uitvoer
  van NKP-opgave "Turing-compiler" }

const
   maxtrans = 100;
   maxband = 100;
   leeg = '~';
   leesset = [ 'a'..'g', 'q', leeg, '0'..'9', ' ' ];

type
   text = file of char;
   string = array[1..52] of char;
   transcomp = record
      { 0..7 <--> 'a'..'g', 8 <--> leeg }
      newstate : integer;
      newsymb, dir  : char;
   end;
   transitie = array[0..8] of transcomp;

var     { globaal }
   transfile, inputfile : text;
   transfie : array[0..maxtrans] of transitie;
   q0,                    { nummer begintoestand }
   nstates : integer;     { aantal toestanten #S }
   band : array[-maxband..maxband] of char;     { de band van de TM }


function fileterm : boolean;
{ checkt of "transfile" termineert door een EOF of een EOLN }

begin
   fileterm := eoln(transfile) or eof(transfile);
end; { fileterm }


function leesregel (var s : string) : boolean;
{ leest een regel van maximaal 51 tekens in LEESSET plus EOLN uit "transfile"
 en retourneert of dit gelukt is; EOLN wordt door '!' vervangen }

var
   ch : char;
   i : integer;

begin
   i := 0;   { I :: i = #ingelezen karakters }
   while not(fileterm) and (i<=51) and (^transfile in leesset) do begin
      i := i+1;
      read(transfile, s[i]);
   end;
   if eoln(transfile) and (i<=51) then begin
      s[i+1] := '!';   { s[i+2..52] ongedefinieerd }
      readln;   { EOLN weglezen }
      leesregel := true;
   end else begin
      leesregel := false;
   end;
end; { leesregel }


function leesq0 : boolean;
{ leest de regel met de beschrijving van de begintoestand
 en retourneert of dit gelukt is }

var
   pos : integer;
   OK : boolean;
   regel : string;

begin
   OK := false;   { I :: OK=helemaal geslaagd }
   if leesregel(regel) then begin
      if (regel[1]='q') and (regel[2]='0') and (regel[3]='=') then begin
         { probeer getal te lezen }
         q0 := 0;
         pos := 4;
         while regel[pos] in ['0'..'9'] do begin
	    q0 : = 10*q0 + ord(regel[pos]) - ord('0');
            pos := pos+1;
         end;
         OK := (regel[pos]='!');
      end;
   end;
   leesq0 := OK;
end; { leesg0 }


function lees3getal (var getal : integer) : boolean;
{ leest een voorgespati"eerd getal van maximaal 3 plaatsen uit "transfile"
 in "getal" en retourneert of dit gelukt is (zoniet ==> getal=0) }

var
   ch : char;
   i : integer;

begin
   i := 0;  { I :: #gelezen tekens }
   { eerst spaties (maximaal 2) weglezen }
   while (i<=2) and (^transfile=' ') do begin
      read(transfile, ch); { ch = ' ' }
      i := i+1;
   end;
   { getal inlezen }
   getal := 0;
   while (i<=3) and (^transfile in ['0'..'9']) do begin
      read(transfile, ch);
      getal := 10*getal + ord(ch) - ord('0');
      i := i+1;
   end;
   { alles goed <==> precies 3 tekens (spaties, cijfers) ingelezen }
   lees3getal := (i=3);
end; { leesgetal }


function leestranstupel (var transtupel : transcomp) : boolean;

var
   ch : char;
   OK : boolean;

begin
   with transtupel do begin
      OK := false;   { I :: OK=helemaal geslaagd }
      if lees3getal(newstate) then begin
         read(ch);
         if ch in ['a'..'g', leeg] then begin
	    newsymb := ch;
	    read(ch);
	    if ch in ['L', 'R', 'h', '.'] then begin
	       dir := ch;
	       OK := true;
	    end;
	 end;
      end;
      leestranstupel := OK;
   end;
end; { leestranstupel }


function leestabelregel : boolean;

var
   n, oldsymb : integer;
   correct, OK : boolean;
   ch : char;

begin
   OK := false;   { I :: OK=helemaal geslaagd }
   { check nummer van de toestand en volgorde toestanden }
   if lees3getal(n) and (n=nstates) then begin
      read(ch);
      if ch=' ' then begin
         correct := true;
         oldsymb := -1;   { I :: oldsymb = #transtupels ingelezen }
         while (oldsymb<8) and correct do begin
            oldsymb := oldsymb+1;
            correct := correct and leestranstupel(transfie[n][oldsymb]);
	    if oldsymb = 8 then begin
	       { na een leeg-symbool '~' moet een EOLN volgen }
	       if eoln(transfile) then begin
		  readln;   { EOLN weglezen }
	       end else begin
		  correct := false;
	       end;
	    end else begin
	       { na elk ander symbool een spatie }
               read(ch);
	       correct := correct and (ch=' ');
	    end;
	 end;
	 OK := correct and eoln(transfile);
      end;
   end;
   leestabelregel := OK;
end; { leestabelregel }


function leestransfie : boolean;
{ leest transitiefunctie in uit "transfile" en geeft
 terug of deze syntactisch correct is }

var
   fout, OK : boolean;
   regel : string;

begin
   reset(transfile);
   OK := false;   { I :: OK=helemaal geslaagd }
   { probeer q0 te lezen }
   if leesq0 then begin
      { eerste regel OK, nu tabellering }
      leesregel(regel);
      if regel=' n  a     b     c     d     f     g    ~    ' then begin
         { tabellering OK, nu transitiefunctie inlezen }
         fout := false;
         nstates := -1;     { I :: #states ingelezen }
         while not(eof(transfile) or fout) and (nstates<maxtrans) do begin
            nstates := nstates + 1;
            fout := leestabelregel;
	 end;
	 OK := (fout and (nstates<=maxtrans));
      end;
   end;
   leestransfie := OK;
end; { leestransfie }


procedure leesinvoer;
{ leest de TM-invoer uit "inputfile" }

var
   ch : char;
   bandpos : integer;

begin
   { leeg de band, d.w.z.: vul de band met leeg-symbolen }
   for bandpos := -maxband to maxband do begin
      band[bandpos] := leeg;
   end;
   { lees tekens vanaf positie 0 naar rechts in }
   bandpos := 0;
   reset(inputfile);
   while not eof(inputfile) do begin
      read(inputfile, band[bandpos]);
      bandpos := bandpos+1;
      end;
   end;
end; { leesinvoer }


function simuleerTM : boolean;
{ simuleert de gegeven TM en
 checkt of die aan de voorwaarden voldoet }

begin
end; { simuleerTM }


begin
   if (not leestransfie) or (q0>nstates) then begin
      writeln('FALSE');
   end else begin
      leesinvoer;     { jury-made ==>? altijd goed }
      writeln(simuleerTM);
   end;
end.
