/*
   generate.c :: input-generator for "Jurassic Park"
   (c) 13/10/'97 by M. Boersma for NKP'97
*/


#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#define max_m 50000

typedef enum { false, true } boolean;
typedef char DNA[max_m +1];

char amino[4] = { 'a', 't', 'c', 'g'  };     /* amino-acids */

FILE *gen;     /* file-pointer of gen-file */
FILE *dna;     /* file-pointer of dna-file */

DNA P;            /* pattern */
int P_length;     /* length of pattern */



void generate_pattern (void)
{
  int seed;
  char ch;
  int i;

#define str_max 64
  char pat_name[str_max];
  FILE *pat;

  fscanf(gen, "%c", &ch);
  switch (ch) {
    case 'r' : {
      fscanf(gen, "%d%d", &P_length, &seed);
      fprintf(dna, "%d\n", P_length);
      srandom((unsigned int)(seed));
      for (i=0; i<P_length; i++) {
	P[i] = amino[random()%4];
	fprintf(dna, "%c", P[i]);
      }
      fprintf(dna, "\n");
    }; break;
    case 'p' : {
      fscanf(gen, "%d %s", &P_length, pat_name);
      fprintf(dna, "%d\n", P_length);
      pat = fopen(pat_name, "r");
      if (pat == NULL) {
	fprintf(stderr, "unable to open pat-file %s...\n", pat_name);
	exit(1);
      } else {
	fscanf(pat, "%s", P);
	fprintf(dna, "%s\n", P);
	fclose(pat);
      }
    }; break;
    default : {
      fprintf(stderr, "corrupt gen-file!\n");
      exit(1);
    };
  }
}


void generate_text (void)
{
  int length;       /* length of text */
  int nentries;     /* number of entries */
  int sublen;       /* length of sub-text */
  int seed;         /* random-seed */
  char ch;
  int i;

  fscanf(gen, " %d %d", &length, &nentries);
  fprintf(dna, "%d\n", length);
  for (; nentries>0; nentries--) {
    fscanf(gen, " %c", &ch);
    switch (ch) {
      case 'r' : {
	/* rubbish */
	fscanf(gen, " %d %d", &sublen, &seed);
	srandom((unsigned int)(seed));
	for (; sublen>0; sublen--) {
	  fprintf(dna, "%c", amino[random()%4]);
	}
      }; break;
      case 'p' : {
	/* copy first "sublen" characters of pattern */
	fscanf(gen, " %d", &sublen);
	for (i=0; i<sublen; i++) {
	  fprintf(dna, "%c", P[i]);
	}
      }; break;
    }
  }
  fprintf(dna, "\n");
}


void generate (void)
{
  int ntexts;
  
  fprintf(stderr, "Generating pattern...");
  generate_pattern();
  fprintf(stderr, "done\n");
  fscanf(gen, " %d\n", &ntexts);
  fprintf(stderr, "#texts=%d\n", ntexts);
  fprintf(dna, "%d\n", ntexts);
  for (; ntexts>0; ntexts--) {
    fprintf(stderr, "Generating text...");
    generate_text();
    fprintf(stderr, "done\n");
  }
}


int main (int argc, char *argv[])
{
  if (argc==3) {
    gen = fopen(argv[1], "r");
    if (gen == NULL) {
      fprintf(stderr, "unable to open gen-file %s...\n", argv[1]);
      exit(1);
    } else {
      dna = fopen(argv[2], "w");
      if (dna == NULL) {
	fprintf(stderr, "unable to open dna-file %s...\n", argv[2]);
      } else {
	/* all OK */
	generate();
	fclose(gen);
	fclose(dna);
      }
    }
  } else {
    fprintf(stderr, "DNA-generator\n");
    fprintf(stderr, "   -- arg 1 : gen-file (src)\n");
    fprintf(stderr, "   -- arg 2 : dna-file (dest)\n");
    exit(1);
  };

  exit(0);
}
